#!/usr/bin/perl -w
###eval 'exec /usr/bin/perl -wS $0 ${1+"$@"}'
###        if $running_under_some_shell;
#
# $Header: /cvs/common/k2/src/ear.pl,v 1.1 2006/06/16 14:03:44 laskait Exp $
#
use strict;
use IO::Socket;
use Getopt::Std;

# command line option variables
use vars qw($opt_p $opt_f $opt_w $opt_e $opt_t $opt_h $opt_m);
# other variables
my($port, $sock, $time, $host, $msgid, $dummy);
my($table, $num, $max, $recsize);
my($elevel, $wlevel);
my($pgw, $ifss, $proc);
my($buffer, $msg);
my(%timeouts, $alarm, $mail);

undef(%timeouts);

getopts('hfp:w:e:t:m:');

if (defined($opt_h)) {
  print <<"EOM";
  $0 -- UDP listener and 'mem' message processor
  Swithces:
    [-p <port>]              port number
    [-f]                     run foreground
    [-w <warning level %>]   warning level for tables
    [-e <error level %>]     error level for tables
    [-t <timeout sec>]       message timeout
    [-m <mail address>]      E-mail address
    [-h]                     help
EOM
  exit 0;
}

# Port where ear is listening by default
$port = defined($opt_p) ? $opt_p : 40000;
warn "[ear] will read port: $port\n";

# Warn level
$wlevel = defined($opt_w) ? $opt_w : 0.80;
warn "[ear] warn level is $wlevel\n";

# Error level
$elevel = defined($opt_e) ? $opt_e : 0.95;
warn "[ear] error level is $elevel\n";

# Timeout level
$alarm = defined($opt_t) ? $opt_t : 15;
warn "[ear] timeout level is $alarm s\n";

# Timeout level
$mail = defined($opt_m) ? $opt_m : undef;
warn "[ear] mail address is $mail\n" if defined $mail;

$sock = IO::Socket::INET->new(LocalAddr => INADDR_ANY,
                              LocalPort => $port,
                              Type      => SOCK_DGRAM,
                              Proto     => 'udp') || die "[ear] cannot create socket $!";

unless (defined($opt_f)) { &daemonise };

open LOG, ">>ear.log" || die "[ear] cannot open ear.log: $!";
autoflush LOG 1;

while (1) {
  eval {
    local $SIG{ALRM} = sub { die "alarm\n" }; # NB: \n required
    alarm $alarm;
    $buffer = <$sock>;            # read single record
    alarm 0;
  };
  if ($@) {
    die unless $@ eq "alarm\n";   # propagate unexpected errors
    foreach $host (keys %timeouts) {
      &check_timeout($host, $alarm, 0);
    }
  } else {
    $time = localtime();
    if ($buffer =~ /^==\>/) {         # first line
      ($dummy, $host, $msgid) = split(/\s+/, $buffer);
      &check_timeout($host, $alarm, 1);
      open FILE, ">>$host" || warn "[ear] cannot open $host log file $!";
      autoflush FILE 1;
      print FILE "At: " . $time . "\n";
      print FILE "From: " . $host . "\n";
      print FILE "Msgid: " . $msgid . "\n";
      $pgw = $ifss = 0;
    } elsif ($buffer =~ /^===/) {     # last line
      if (!$pgw) {
        $msg = "$time --> ERROR pgw exited on $host\n";
        print LOG $msg;
        &send_mail($msg, $mail) if defined $mail;
      }
      if (!$ifss) {
        $msg = "$time --> ERROR ifss exited on $host\n";
        print LOG $msg;
        &send_mail($msg, $mail) if defined $mail;
      }
      print FILE $buffer;
    } elsif ($buffer =~ /^order|^trade|^negdeal|^orderentry|^badreplymsg/) { # growing tables
      ($table, $num, $max, $recsize) = split(/\s+/, $buffer);
      if (($num / $max) > $elevel) {
        $msg = "$time --> ERROR $table is almost full, num: $num, max: $max\n";
        print LOG  $msg;
        &send_mail($msg, $mail) if defined $mail;
      } elsif (($num / $max) > $wlevel) {
        $msg = "$time --> WARNING $table is become full, num: $num, max: $max\n";
        print LOG $msg;
        &send_mail($msg, $mail) if defined $mail;
      }
      print FILE $buffer;
    } elsif ($buffer =~ /^[0-9]+/) { # process info
      $proc = (split(/\s+/, $buffer))[2];
      if ($proc eq "pgw")  { $pgw = 1  };
      if ($proc eq "ifss") { $ifss = 1 };
      print FILE $buffer;
    } else {              # other stuff
      print FILE $buffer;
    }
  }
}

sub send_mail {
  my $msg = shift;
  my $mail = shift;

  open(MAIL, "|mail $mail");
  print MAIL $msg;
  close MAIL;
}

sub check_timeout {
  my $host = shift;
  my $alarm = shift;
  my $m = shift;
  my($tt, $time, $msg);

  # message timeout check
  $tt = time();
  $time = localtime();
  if (!exists($timeouts{$host})) {
    $timeouts{$host} = $tt;
  }
  if ($tt - $timeouts{$host} > $alarm * 1.2) {
    $msg = sprintf "$time --> WARNING $host has sent %d seconds before!\n", $tt - $timeouts{$host};
    print LOG $msg;
    &send_mail($msg, $mail) if defined $mail;
  }
  if ($m) { # update only if message arrived
    $timeouts{$host} = $tt;
  }
}

sub daemonise {
  my($child);

  $child = fork;
  if ($child == 0) {     # child
    setpgrp();
  } elsif ($child > 0) { # parent
    exit(0);
  } else {               # error
    die "[ear] cannot fork $!\n";
  }
}
