/*****************************************************************************
         (c) Copyright Effice Mrnki Iroda Kft. 1999.

Ez az llomny kizrolag az Effice Kft. (2083 Solymr Toldi u. 21.) tulajdona,
kizrolag csak az Effice Kft. engedlyvel, az engedlyben meghatrozott clra
s mdon hasznlhat fel. A program ingyenesen trtnt tadsra, mkdsrt 
az Effice Kft. semmilyen felelssget nem vllal.

Demo program about using ifsc functions.
*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "ifsapi.h"
#include "ifsdefs.h"
#include "ifsutil.h"

/* static char Id[] = ""; */
static char prt_buf[0x8000];

static char * print_message(char * buf, char * msg, int len);
void usage( void);
int set_test_mode(IFSC_HDL handle, int mode);

#define CREATE_TEST_DATA 1
#define READ_TEST_DATA 0

#ifndef WIN32
#  define SLEEP(x) sleep(x)
#else
#  define SLEEP(x)
#endif

/*+++++++++++++++++++++++++++++++++++++++++
Prints usage information.
+++++++++++++++++++++++++++++++++++++++++*/
void usage( void){
  fprintf(stderr, "Usage [-hhost] [-sservice] [-uuser] [-ppwd]\n");
  fprintf(stderr, "  -hhost     host name of server\n");
  fprintf(stderr, "  -sservice  service name of server\n");
  fprintf(stderr, "  -uuser     user name\n");
  fprintf(stderr, "  -pwd       password\n");
  fprintf(stderr, "If no host, service, user, pwd defined then \n");
  fprintf(stderr, "  value of IFSHOST, IFSSERVICE, IFSUSER, IFSPWD environment used\n");
  fprintf(stderr, "\n");
  exit(1);    
}

/*+++++++++++++++++++++++++++++++++++++++++
Print a message.

char * print_message Points to the message.

char * buf Pointer to the print buffer(output).

char * msg Pointer to message (input).

int len Length of the input.
+++++++++++++++++++++++++++++++++++++++++*/
static char * print_message(char * buf, char * msg, int len)
{
  int i;
  for (i = 0; i < len; i++){
    if (msg[i]) 
      buf[i] = msg[i];
    else
      buf[i] = '|'; 
  }
  buf[i]='\0';
  return buf;
}
/*+++++++++++++++++++++++++++++++++++++++++
Print field change message.

char * print_field_chg Points to the message.

char * buf Pointer to the print buffer(output).

char * msg Pointer to message (input).

int len Length of the input.
+++++++++++++++++++++++++++++++++++++++++*/
static char * print_field_chg(char * buf, char * msg, int len , int mmts_type)
{
  int i, pos =0, data_len;
  char field_code, *data;

  sprintf(buf, "%s|", msg); /* record id */
  for (i = IFS_OK; i == IFS_OK;) { 
  	  if ( mmts_type == MMTS_TYPE_1)
		  i = ifs_get_next_field(msg, len, &pos, &field_code, &data, &data_len);
	  else
		  i = ifs_get_next_field_mmts2(msg, len, &pos, &field_code, &data, &data_len);
	  if (i == IFS_OK) {
		  sprintf(buf + strlen(buf), " %d:%s|", field_code, data); /* field codes and field data */
	  }
  }        
  return buf;
}

/*+++++++++++++++++++++++++++++++++++++++++
Queries the defined table while change received, and prints into stdout.

int get_table Error status of query.

IFSC_HDL handle Pointer to the connection handler.

int table_code Code of the table.
+++++++++++++++++++++++++++++++++++++++++*/
int get_table(IFSC_HDL handle, int table_code)
{
  int s, len, seqno;
  char * recvb;

  while ((s = ifsc_get_next_record(handle, &recvb, &len, table_code, &seqno)) == 0) {
    fprintf(stderr, "ifsc_get_next_record table: %d : status: %d, len %d, seqno: %d\n\n",table_code, s, len, seqno); 
    fflush(stderr);
    fprintf(stdout, "%s\n", print_message(prt_buf, recvb, len));
    fflush(stdout);
  }
  if (s != IFS_NOMORE) {
    fprintf(stderr, "Error at ifsc_get_next_record: %s \n", ifsc_get_last_errmsg(handle));
    fflush(stderr);
  }

  return s;
}

/*+++++++++++++++++++++++++++++++++++++++++
Queries the secboard table in field mode, and prints into stdout.

int get_secboard_field_more Error status of query.

IFSC_HDL handle Pointer to the connection handler.
+++++++++++++++++++++++++++++++++++++++++*/
int get_secboard_field_mode( IFSC_HDL handle , int mmts_type)
{
  int s, len, seqno;
  char * recvb;

  while ((s = ifsc_get_next_idx(handle, &recvb, &len, IFS_T_SECBOARD, &seqno)) == 0) {
    fprintf(stderr, "ifsc_get_next_idx table: SECBOARD : status: %d, len %d, seqno: %d\n\n", s, len, seqno); 
    fflush(stderr);
    fprintf(stdout, "%s\n", print_message(prt_buf, recvb, len));
    fflush(stdout);
  }
  if (s != IFS_NOMORE) {
    fprintf(stderr, "Error at ifsc_get_next_idx: %s \n", ifsc_get_last_errmsg(handle));
    fflush(stderr);
  }

  while ((s = ifsc_get_next_field_chg(handle, &recvb, &len, IFS_T_SECBOARD, &seqno)) == 0) {
    fprintf(stderr, "ifsc_get_next_field_chg table: SECBOARD : status: %d, len %d, seqno: %d\n\n", s, len, seqno); 
    fflush(stderr);
    fprintf(stdout, "%s\n", print_field_chg(prt_buf, recvb, len, mmts_type));
    fflush(stdout);
  }
  if (s != IFS_NOMORE) {
    fprintf(stderr, "Error at ifsc_get_next_idx: %s \n", ifsc_get_last_errmsg(handle));
    fflush(stderr);
  }

  return s;
}

/*+++++++++++++++++++++++++++++++++++++++++
Send an orderentry, and prints status into stdout.

int send_order Error status of ifsc_orderentry.

IFSC_HDL handle Pointer to the connection handler.

char * securityid Name of the security.

int price Price of orderentry.

int quantity Quantity of orderentry.

int mmts_type Type of the MMTS where server connected (1 or 2).
+++++++++++++++++++++++++++++++++++++++++*/
int send_order(IFSC_HDL handle, char * securityid, int price, int quantity, int mmts_type)
{
  int s, len, orderid;
  char buf[1024];

  /* prepare an orderentry record */
  if (mmts_type == MMTS_TYPE_1) {
  
    len = ifs_set_ids(buf, "EFFICE");
    len += ifs_set_char(buf + len, 'B');
    len += ifs_set_string(buf + len, IFS_ORDERMETHOD_LEN, "01000000000000000100000000010000");
    len += ifs_set_string(buf + len, IFS_BOARDID_LEN, "FR");
    len += ifs_set_ids(buf + len, securityid);
    len += ifs_set_int(buf + len, price);
    len += ifs_set_int(buf + len, 0);
    len += ifs_set_int(buf + len, quantity);
    len += ifs_set_int(buf + len, 0);
    len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, "demo");
    len += ifs_set_int(buf + len, 20000202);
    len += ifs_set_int(buf + len, 0);
    len += ifs_set_int(buf + len, 0);
    len += ifs_set_int(buf + len, 20000203);
    len += ifs_set_int(buf + len, 0);
    len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, "bos ref");
    
  } else if (mmts_type == MMTS_TYPE_2) {
    struct fix_real fr;
    
    len = ifs_set_ids(buf, "EFFICE");
    len += ifs_set_enum(buf + len, 0); /* buysell */
    len += ifs_set_enum(buf + len, 0); /* ordertype */
    len += ifs_set_enum(buf + len, 3); /* duration, 3 = day */
    len += ifs_set_enum(buf + len, 0); /* purgeonlogoff */
    len += ifs_set_enum(buf + len, 0); /* allowsoftqtylimit */
    len += ifs_set_enum(buf + len, 0); /* allowsoftpricelimit */
    len += ifs_set_enum(buf + len, 0); /* positiontype */
    len += ifs_set_enum(buf + len, 0); /* isprivate */
    len += ifs_set_string(buf + len, IFS_BOARDID_LEN, "HRVB");
    len += ifs_set_string(buf + len, IFS_SEC_CODE_LEN, securityid);
    fr.value = price;
    fr.decimals = 0;
    len += ifs_set_fix_real(buf + len, fr); /* price */
    fr.value = 0;
    fr.decimals = NOT_DEFINED;
    len += ifs_set_fix_real(buf + len, fr); /* yield */
    len += ifs_set_int(buf + len, quantity);
    len += ifs_set_int(buf + len, quantity); /* Visible qty */
    len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, "demo");
    len += ifs_set_int(buf + len, 0); /* ExpTime date */
    len += ifs_set_int(buf + len, 0); /* ExpTime time */
    fr.value = 0;
    fr.decimals = NOT_DEFINED;
    len += ifs_set_fix_real(buf + len, fr); /* triggerprice */
    len += ifs_set_string(buf + len, IFS_NAME_LEN, "");
    len += ifs_set_int(buf + len, 0); /* minfillqty */
    len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, "bos ref"); /* internalref */    
  
  } else {
    fprintf(stderr, "Invalid mmts type %d", mmts_type);
    return -1;
  }
  
  /* sending an orderentry record */
  if ((s = ifsc_orderentry(handle, IFS_ACTION_ORDER_ADD, buf, len, &orderid)) == IFS_OK) {
    fprintf (stderr, " order_add (orderid %d) sent\n", orderid);
    fflush(stderr);
  } 
  else if (s == IFS_NOMORE) {
    fprintf (stderr, "Order add (%s, %d, %d) not found in test data \n", securityid, price, quantity);
    fflush(stderr);
  } else {
    fprintf (stderr, " error at order_add - %s\n", ifsc_get_last_errmsg(handle));
    fflush(stderr);
  }

  if (s == IFS_OK){
    if ((s = ifsc_orderentry_status_chg(handle, orderid, IFS_ORDER_CONFIRMED) == IFS_OK)){
      fprintf(stderr, "order status changed for %d order\n", orderid);
      fflush(stderr);
    }
    else if (s == IFS_NOMORE) {
      fprintf (stderr, "Orderentry status change (%s) not found in test data \n", orderid);
      fflush(stderr);
    } else {
      fprintf (stderr, " error at order_add - %s\n", ifsc_get_last_errmsg(handle));
      fflush(stderr);
    }
  }

  return s;
}

/*+++++++++++++++++++++++++++++++++++++++++
Send an order amendment, and prints status into stdout.

int amend_order Error status of ifsc_orderentry.

IFSC_HDL handle Pointer to the connection handler.

char *orderNo Order number.

int price New Price of orderentry.

int quantity New quantity of orderentry.

int mmts_type Type of MMTS where server connected.
+++++++++++++++++++++++++++++++++++++++++*/
int amend_order(IFSC_HDL handle, char *orderNo, int price, int quantity, int mmts_type)
{
  int s, len, orderid;
  char buf[1024];

  /* prepare an orderentry record */
  if (mmts_type == MMTS_TYPE_1) {
  
    len = ifs_set_int(buf, atoi(orderNo));
    len += ifs_set_ids(buf + len, "EFFICE");
    len += ifs_set_string(buf + len, IFS_ORDERMETHOD_LEN, "00000000000000000000000000010000");
    len += ifs_set_int(buf + len, price);
    len += ifs_set_int(buf + len, 0);
    len += ifs_set_int(buf + len, quantity);
    len += ifs_set_int(buf + len, 0);
    len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, "demo");
    len += ifs_set_int(buf + len, 20000202);
    len += ifs_set_int(buf + len, 0);
    len += ifs_set_int(buf + len, 0);
    len += ifs_set_int(buf + len, 20000203);
    len += ifs_set_int(buf + len, 0);
    len += ifs_set_ids(buf + len, "");
    len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, "bos ref");

  } else if (mmts_type == MMTS_TYPE_2) {
    struct fix_real fr;

    len = ifs_set_string(buf, IFS_ORDERNO_LEN, orderNo);
    len += ifs_set_int(buf + len, -1); /* OrdNoSpeedIdx */
    len += ifs_set_ids(buf + len, "EFFICE"); /* TrdAcc */
    len += ifs_set_enum(buf + len, 3); /* duration, 3 = day */
    len += ifs_set_enum(buf + len, 0); /* AllowSoftQtyLimit */
    len += ifs_set_enum(buf + len, 0); /* AllowSoftPriceLimit */
    len += ifs_set_enum(buf + len, 0); /* PositionType */
    fr.value = price;
    fr.decimals = 0;
    len += ifs_set_fix_real(buf + len, fr);  /* price */
    fr.value = 0;
    fr.decimals = NOT_DEFINED;
    len += ifs_set_fix_real(buf + len, fr);  /* yield */
    len += ifs_set_int(buf + len, quantity);
    len += ifs_set_int(buf + len, quantity); /* visible Qty */
    len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, "demo");
    len += ifs_set_int(buf + len, 0);        /* Expdate */
    len += ifs_set_int(buf + len, 0);        /* ExpTime */
    len += ifs_set_fix_real(buf + len, fr);  /* Trigger price */
    len += ifs_set_int(buf + len, 0);        /* Minfillqty */
    len += ifs_set_string(buf + len, IFS_NAME_LEN, ""); /* TradeRef */
    len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, "bos ref");

  } else {
    fprintf(stderr, "Invalid mmts type %d", mmts_type);
    return -1;
  }

  /* sending an orderentry record */
  if ((s = ifsc_orderentry(handle, IFS_ACTION_ORDER_AMEND, buf, len, &orderid)) == IFS_OK) {
    fprintf (stderr, " order amend (orderid %d) sent\n", orderid);
    fflush(stderr);
  } 
  else if (s == IFS_NOMORE) {
    fprintf (stderr, "Order amend (orderno: %d) not found in test data \n", orderNo);
    fflush(stderr);
  } else {
    fprintf (stderr, " error at order amend - %s\n", ifsc_get_last_errmsg(handle));
    fflush(stderr);
  }

  if (s == IFS_OK){
    if ((s = ifsc_orderentry_status_chg(handle, orderid, IFS_ORDER_CONFIRMED) == IFS_OK)){
      fprintf(stderr, "order status changed for %d order\n", orderid);
      fflush(stderr);
    }
    else if (s == IFS_NOMORE) {
      fprintf (stderr, "Orderentry status change (%s) not found in test data \n", orderid);
      fflush(stderr);
    } else {
      fprintf (stderr, " error at order_add - %s\n", ifsc_get_last_errmsg(handle));
      fflush(stderr);
    }
  }

  return s;
}

/*+++++++++++++++++++++++++++++++++++++++++
Send an order withdraw, and prints status into stdout.

int withdraw_order Error status of ifsc_orderentry.

IFSC_HDL handle Pointer to the connection handler.

char * orderNo Order number.

char * securityid Name of the security.

int mmts_type Type of the MMTS where server connected.
+++++++++++++++++++++++++++++++++++++++++*/
int withdraw_order(IFSC_HDL handle, char * orderNo, char * securityid, int mmts_type)
{
  int s, len, orderid;
  char buf[1024];

  /* prepare an orderentry record */
  if (mmts_type == MMTS_TYPE_1) {

    len = ifs_set_int(buf, atoi(orderNo));
    len += ifs_set_ids(buf + len, " ");
    len += ifs_set_char(buf + len, ' ');
    len += ifs_set_string(buf + len, IFS_BOARDID_LEN, " ");
    len += ifs_set_ids(buf + len, securityid);
    len += ifs_set_int(buf + len, 0);
    len += ifs_set_int(buf + len, 0);
    len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, "demo");
    len += ifs_set_char(buf + len, 'A');
    len += ifs_set_char(buf + len, '=');
    len += ifs_set_ids(buf + len, " ");
    len += ifs_set_ids(buf + len, " ");
    len += ifs_set_int(buf + len, 0);
    len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, "bos ref");

  } else if (mmts_type == MMTS_TYPE_2) {
    struct fix_real fr;

    len = ifs_set_string(buf, IFS_ORDERNO_LEN, orderNo);
    len += ifs_set_int(buf + len, -1);    /* OrdNoSpeedIdx */
    len += ifs_set_ids(buf + len, " ");  /* TrdAcc */
    len += ifs_set_enum(buf + len, 2);    /* BuySell, 2 = buyorsell */
    len += ifs_set_string(buf + len, IFS_BOARDID_LEN, " "); /* boardid */
    len += ifs_set_ids(buf + len, " ");  /* InstrId */
    len += ifs_set_string(buf + len, IFS_SEC_CODE_LEN, securityid);
    fr.value = 0;
    fr.decimals = NOT_DEFINED;
    len += ifs_set_fix_real(buf + len, fr);   /* price */
    len += ifs_set_fix_real(buf + len, fr);   /* yield */
    len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, "demo");
    len += ifs_set_enum(buf + len, 0);   /* OpCode, 0 = AND */
    len += ifs_set_enum(buf + len, 0);   /* PopCode, 0 = EQ */
    len += ifs_set_string(buf + len, IFS_ORDERNO_LEN, " "); /* MultilegOrdNo */
    len += ifs_set_int(buf + len, 0);    /* MultilegSpeedIdx */
    len += ifs_set_ids(buf + len, " ");  /* UserId */
    len += ifs_set_ids(buf + len, " ");  /* FirmId */
    len += ifs_set_int(buf + len, 0);    /* OrderDate date */
    len += ifs_set_int(buf + len, 0);    /* OrderDate time */
    len += ifs_set_enum(buf + len, 0);   /* MarketMaker */
    len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, "bos ref");

  } else {
    fprintf(stderr, "Invalid mmts type %d", mmts_type);
    return -1;
  }

  /* sending an orderentry record */
  if ((s = ifsc_orderentry(handle, IFS_ACTION_ORDER_WITHDRAW, buf, len, &orderid)) == IFS_OK) {
    fprintf (stderr, " order withdraw (orderid %d) sent\n", orderid);
    fflush(stderr);
  } 
  else if (s == IFS_NOMORE) {
    fprintf (stderr, "Order withdraw (orderno: %d) not found in test data \n", orderNo);
    fflush(stderr);
  } else {
    fprintf (stderr, " error at order withdraw - %s\n", ifsc_get_last_errmsg(handle));
    fflush(stderr);
  }

  if (s == IFS_OK){
    if ((s = ifsc_orderentry_status_chg(handle, orderid, IFS_ORDER_CONFIRMED) == IFS_OK)){
      fprintf(stderr, "order status changed for %d order\n", orderid);
      fflush(stderr);
    }
    else if (s == IFS_NOMORE) {
      fprintf (stderr, "Orderentry status change (%s) not found in test data \n", orderid);
      fflush(stderr);
    } else {
      fprintf (stderr, " error at order_add - %s\n", ifsc_get_last_errmsg(handle));
      fflush(stderr);
    }
  }

  return s;
}

/*+++++++++++++++++++++++++++++++++++++++++
Sets orderbook watch to given secboard, queries data of that anf prints into stdout.

int get_ob Error status of ifsc_get_next_orderbook.

IFSC_HDL handle Pointer to the connection handler.

char * secboardid SecboardId.
+++++++++++++++++++++++++++++++++++++++++*/
int get_ob(IFSC_HDL handle, char * secboardid)
{
  int s, len, seqno = 0;
  char * recvb;

  /* adding watch */
  if ((s = ifsc_orderbook_conf(handle, secboardid, IFS_SWITCH_ON)) == IFS_OK) {
    fprintf (stderr, " orderbook watch added for %s\n", secboardid);
    fflush(stderr);
  }  
  else if (s == IFS_NOMORE){
    fprintf (stderr, "Orderbook watch %s not found in test data!\n", secboardid);
    fflush(stderr);
  }
  else {
    fprintf (stderr, "%s -- %s\n", ifsc_get_last_errmsg(handle), secboardid);
    fflush(stderr);
  }

  SLEEP(4);

  if ((s = ifsc_get_next_orderbook(handle, &recvb, &len, secboardid)) >= 0) {
    fprintf (stderr, "orderbook queried for %s\n", secboardid);
    fflush(stderr);
    if (!s) {
      fprintf(stdout, "%s\n", print_message(prt_buf, recvb, len)); 
      fflush(stdout);
    }
  }  
  else
  {
    fprintf (stderr, "orderbook querying failed for %s - %s\n", secboardid, ifsc_get_last_errmsg(handle));
    fflush(stderr);
  }

  return s;
}

/*+++++++++++++++++++++++++++++++++++++++++
Gets orderbook list, and prints into stdout.

int get_ob_list Error status of ifsc_get_orderbook_list.

IFSC_HDL handle Pointer to the connection handler.
+++++++++++++++++++++++++++++++++++++++++*/
int get_ob_list(IFSC_HDL handle)
{
  int i, s, len;
  char * recvb;

  if ((s = ifsc_get_orderbook_list(handle, &recvb, &len)) == 0) { 
    for(i=0; i < len; i++)
      if (recvb[i] == 0) recvb[i] = '\n';
    fprintf(stdout, "%*.*s", len, len, recvb);
    fflush(stdout);
  }
  else if (s == IFS_NOMORE){
    fprintf (stderr, "Orderbook_list not found in test data!\n");
    fflush(stderr);
  }
  else {
    fprintf (stderr, "%s\n", ifsc_get_last_errmsg(handle));
    fflush(stderr);
  }

  return s;
}

/*+++++++++++++++++++++++++++++++++++++++++
Sets market by price watch to given secboard, queries data of that anf prints into stdout.

int get_mbp Error status of ifsc_get_next_marketbyprx.

IFSC_HDL handle Pointer to the connection handler.

char * secboardid SecboardId.
+++++++++++++++++++++++++++++++++++++++++*/
int get_mbp(IFSC_HDL handle, char * secboardid)
{
  int s, len, seqno = 0;
  char * recvb;

  /* adding watch */
  if ((s = ifsc_marketbyprx_conf(handle, secboardid, IFS_SWITCH_ON)) == IFS_OK) {
    fprintf (stderr, " market by price watch added for %s\n", secboardid);
    fflush(stderr);
  }  
  else if (s == IFS_NOMORE){
    fprintf (stderr, "market by price watch %s not found in test data!\n", secboardid);
    fflush(stderr);
  }
  else {
    fprintf (stderr, "%s -- %s\n", ifsc_get_last_errmsg(handle), secboardid);
    fflush(stderr);
  }

  SLEEP(4);

  if ((s = ifsc_get_next_marketbyprx(handle, &recvb, &len, secboardid)) >= 0) {
    fprintf (stderr, "market by price queried for %s\n", secboardid);
    fflush(stderr);
    if (!s) {
      fprintf(stdout, "%s\n", print_message(prt_buf, recvb, len)); 
      fflush(stdout);
    }
  }  
  else
  {
    fprintf (stderr, "market by price querying failed for %s - %s\n", secboardid, ifsc_get_last_errmsg(handle));
    fflush(stderr);
  }

  return s;
}

/*+++++++++++++++++++++++++++++++++++++++++
Gets market by price list, and prints into stdout.

int get_mbp_list Error status of ifsc_get_marketbyprx_list.

IFSC_HDL handle Pointer to the connection handler.
+++++++++++++++++++++++++++++++++++++++++*/
int get_mbp_list(IFSC_HDL handle)
{
  int i, s, len;
  char * recvb;

  if ((s = ifsc_get_marketbyprx_list(handle, &recvb, &len)) == 0) { 
    for(i=0; i < len; i++)
      if (recvb[i] == 0) recvb[i] = '\n';
    fprintf(stdout, "%*.*s", len, len, recvb);
    fflush(stdout);
  }
  else if (s == IFS_NOMORE){
    fprintf (stderr, "marketbyprx_list not found in test data!\n");
    fflush(stderr);
  }
  else {
    fprintf (stderr, "%s\n", ifsc_get_last_errmsg(handle));
    fflush(stderr);
  }

  return s;
}
/*+++++++++++++++++++++++++++++++++++++++++
Queries all tables, orderbook, market by price, sends orderentries, 
orderbook, market by price configuration.

int main Tradition.

int argc Number of command line parameters.

char *argv[] Points to an array that contains the command line parameters.
+++++++++++++++++++++++++++++++++++++++++*/
int main (int argc, char *argv[])
{
  int tradeid, pid, i, mmts_type;
  char * host = NULL;
  char * service = NULL;
  char * user = NULL;
  char * pwd = NULL;
  int create = 0;
  IFSC_HDL handle;
  char sec1[IFS_IDS_LEN + IFS_BOARDID_LEN], sec2[IFS_IDS_LEN + IFS_BOARDID_LEN], secboard1[IFS_IDS_LEN + IFS_BOARDID_LEN], secboard2[IFS_IDS_LEN + IFS_BOARDID_LEN];
  char amend_orderno[IFS_ORDERNO_LEN], withdraw_orderno[IFS_ORDERNO_LEN];
  int price1, price2, price_amend;
 
  host = getenv("IFSHOST");
  service = getenv("IFSSERVICE");
  user = getenv("IFSUSER");
  pwd = getenv("IFSPWD");


  for (i = 1; i < argc; i++){
    if (argv[i][0] == '-'){
      switch(argv[i][1]){
        case 'h':
          host = argv[i]+2;
          break;
        case 's':
          service = argv[i]+2;
          break;
        case 'u':
          user = argv[i]+2;
          break;
        case 'p':
          pwd = argv[i]+2;
          break;
	case 'c':
	  create = 1;
          break;
      }
    } else {
        usage();
    }  
  }

  if (!host) {
    fprintf(stderr, "The host is not defined!\n");
    usage();
  }

  if (!service) {
    fprintf(stderr, "The service is not defined!\n");
    usage();
  }

  if (!user) {
    fprintf(stderr, "The user is not defined!\n");
    usage();
  }

  if (!pwd) {
    fprintf(stderr, "The pwd is not defined!\n");
    usage();
  }

  /* Creating */
  if (ifsc_create(&handle, host, service)) {
    fprintf(stderr, "Error at creation: %s\n", ifsc_get_last_errmsg(handle));
    exit(1);
  }

  if (create)
    set_test_mode(handle, CREATE_TEST_DATA);

  /* Connecting */
  if (ifsc_connect(handle, user, pwd, &tradeid, &pid, &mmts_type)) {
    fprintf(stderr, "Error at connection: %s\n", ifsc_get_last_errmsg(handle));
    exit(1);
  }
  
  if (mmts_type == MMTS_TYPE_1) {
    strcpy(sec1, "MOL");
    strcpy(sec2, "MTELEKOM");
    strcpy(secboard1, "0FRMOL");
    strcpy(secboard2, "0FRMTELEKOM");
    strcpy(amend_orderno, "1");
    strcpy(withdraw_orderno, "9");
    price1 = 20350;
    price2 = 720;
    price_amend = 5010;
  } else if (mmts_type == MMTS_TYPE_2) {
    strcpy(sec1, "MOL0103");
    strcpy(sec2, "MAT0103");
    strcpy(secboard1, "HRVBMOL0103");
    strcpy(secboard2, "HRVBMAT0103");
    strcpy(amend_orderno, "20001130-000000000001");
    strcpy(withdraw_orderno, "20001130-000000000009");
    price1 = 4100;
    price2 = 10000;
    price_amend = 7210;
  } else {
    fprintf(stderr, "Invalid MMTS type: %d!", mmts_type);
    exit(1);
  }

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "MARKET data:\n");
  fflush(stderr);
  if (get_table(handle, IFS_T_MARKET) < IFS_OK) 
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "INSTR data:\n");
  fflush(stderr);
  if (get_table(handle, IFS_T_INSTR) < IFS_OK) 
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "SECTOR data:\n");
  fflush(stderr);
  if (get_table(handle, IFS_T_SECTOR) < IFS_OK) 
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "BOARD data:\n");
  fflush(stderr);
  if (get_table(handle, IFS_T_BOARD) < IFS_OK) 
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "SECBOARD data:\n");
  fflush(stderr);
  if (get_table(handle, IFS_T_SECBOARD) < IFS_OK) 
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "FIRM data:\n");
  fflush(stderr);
  if (get_table(handle, IFS_T_FIRM) < IFS_OK) 
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "USER data:\n");
  fflush(stderr);
  if (get_table(handle, IFS_T_USER) < IFS_OK) 
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "ORDER data:\n");
  fflush(stderr);
  if (get_table(handle, IFS_T_ORDER) < IFS_OK) 
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "TRADE data:\n");
  fflush(stderr);
  if (get_table(handle, IFS_T_TRADE) < IFS_OK) 
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "NEGDEAL data:\n");
  fflush(stderr);
  if (get_table(handle, IFS_T_NEGDEAL) < IFS_OK) 
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "AUDITEVENT data:\n");
  fflush(stderr);
  if (get_table(handle, IFS_T_AUDITEVENT) < IFS_OK) 
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "PRICEPARAM data:\n");
  fflush(stderr);
  if (get_table(handle, IFS_T_PRICEPARAM) < IFS_OK) 
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "SECBOARD field change data:\n");
  fflush(stderr);
  if (get_secboard_field_mode(handle , mmts_type) < IFS_OK)
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "New order for %s:\n", sec1);
  fflush(stderr);
  send_order(handle, sec1, price1, 1, mmts_type);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "New order for %s:\n", sec2);
  fflush(stderr);
  send_order(handle, sec2, price2, 14, mmts_type);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "Order amend for orderno %s:\n", amend_orderno);
  fflush(stderr);
  amend_order(handle, amend_orderno, price_amend, 22, mmts_type);
  SLEEP(4);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "Order withdraw for orderno %s:\n", withdraw_orderno);
  fflush(stderr);
  withdraw_order(handle, withdraw_orderno, "", mmts_type);
  SLEEP(4);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "Order withdraw for %s:\n", sec1);
  fflush(stderr);
  withdraw_order(handle, "", sec1, mmts_type);
  SLEEP(4);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "ORDERENTRY data:\n");
  fflush(stderr);
  if (get_table(handle, IFS_T_ORDERENTRY) < IFS_OK) 
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "%s orderbook:\n", sec1);
  fflush(stderr);
  if (get_ob(handle, secboard1) < IFS_OK)
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "%s orderbook:\n", sec2);
  fflush(stderr);
  if (get_ob(handle, secboard2) < IFS_OK)
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "Watched orderbooks:\n");
  fflush(stderr);
  if (get_ob_list(handle) < IFS_OK)
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "%s market by price:\n", sec1);
  fflush(stderr);
  if (get_mbp(handle, secboard1) < IFS_OK)
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "%s market by price:\n", sec2);
  fflush(stderr);
  if (get_mbp(handle, secboard2) < IFS_OK)
    exit(1);

  fprintf(stderr, "-------------------------------------------------------------------------------\n");
  fprintf(stderr, "Watched market by price:\n");
  fflush(stderr);
  if (get_mbp_list(handle) < IFS_OK)
    exit(1);

  if (ifsc_disconnect(handle)) {
    fprintf(stderr, "Error at disconnect: %s\n", ifsc_get_last_errmsg(handle));
    fflush(stderr);
    exit(1);
  }
  return 0;
}
