/****************************************
Ez az llomny kizrolag az Effice Kft. (2083 Solymr Toldi u. 21.) tulajdona,
kizrolag csak az Effice Kft. engedlyvel, az engedlyben meghatrozott clra
s mdon hasznlhat fel. A program ingyenesen trtnt tadsra, mkdsrt 
az Effice Kft. semmilyen felelssget nem vllal.

Interface client for socket based communication.
Example program for query market by order or market by price
****************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifndef WIN32
#  include <unistd.h>
#else
#  include <wtypes.h>
#  include <winbase.h>
#endif

#include "ifsapi.h"
#include "ifsdefs.h"
#include "ifsutil.h"

static char Id[] = "get_ob ";

        /* Prototypes */
int main (int argc, char *argv[]);
static char * print_message(char * buf, char * msg, int len);
void usage( void);

#ifdef WIN32
#  define SLEEP(x) Sleep(x * 1000)
#else
#  define SLEEP(x) sleep(x)
#endif

/*+++++++++++++++++++++++++++++++++++++++++
Prints usage information.
+++++++++++++++++++++++++++++++++++++++++*/
void usage( void){
  fprintf(stderr, "Usage <secboardid> [-c] [-m] [-t] [-hhost] [-sservice] [-uuser] [-ppwd]\n");
  fprintf(stderr, "  -c		continue \n");
  fprintf(stderr, "  -m         market by price querying\n");
  fprintf(stderr, "  -t 	trace\n");
  fprintf(stderr, "  -hhost     host name of server\n");
  fprintf(stderr, "  -sservice  service name of server\n");
  fprintf(stderr, "  -uuser     user name\n");
  fprintf(stderr, "  -pwd       password\n");
  fprintf(stderr, "If no host, service, user, pwd defined then \n");
  fprintf(stderr, "  value of IFSHOST, IFSSERVICE, IFSHOST, IFSPWD environment used\n");
  exit(1);    
}

/*+++++++++++++++++++++++++++++++++++++++++
Print a message.

char * print_message Points to the output string.

char * buf Pointer to the print buffer.

char * msg Pointer to message.

int len Length of the input.
+++++++++++++++++++++++++++++++++++++++++*/
static char * print_message(char * buf, char * msg, int len)
{
  int i;
  for (i = 0; i < len; i++){
    if (msg[i]) 
      buf[i] = msg[i];
    else
      buf[i] = '|'; 
  }
  buf[i]='\0';
  return buf;
}
/*+++++++++++++++++++++++++++++++++++++++++
Socket based communication interface client. Clients can connect into this server to 
query orderbook and market by price informations.

int main Tradition.

int argc Number of command line parameters.

char *argv[] Points to an array that contains the command line parameters.
+++++++++++++++++++++++++++++++++++++++++*/
int main (int argc, char *argv[])
{
  int i, s, len, trace = 0, cont = 0, tradeid, pid, marketbyprx = 0, mmts_type;
  char *buff;
  char ptr_buf[0x8000];
  char * secboardid = NULL;
  char * host = NULL;
  char * service = NULL;
  char * user = NULL;
  char * pwd = NULL;
  IFSC_HDL handle;

  host = getenv("IFSHOST");
  service = getenv("IFSSERVICE");
  user = getenv("IFSUSER");
  pwd = getenv("IFSPWD");


  for (i = 1; i < argc; i++){
    if (argv[i][0] == '-'){
      switch(argv[i][1]){
        case 't': 
          trace = 1; 
          break;
        case 'h':
          host = argv[i]+2;
          break;
        case 's':
          service = argv[i]+2;
          break;
        case 'u':
          user = argv[i]+2;
          break;
        case 'p':
          pwd = argv[i]+2;
          break;
        case 'c': 
          cont = 1; 
          break;
        case 'm': 
          marketbyprx = 1; 
          break;
      }
    } else {
      secboardid = argv[i];
    }
  }

  if (trace) fprintf (stderr, "%s Compiled: %s %s\n", Id, __DATE__, __TIME__);

  if (!secboardid) {
    fprintf(stderr, "Not enaugh parameter!\n");
    usage();
  }

  if (!host) {
    fprintf(stderr, "The host is not defined!\n");
    usage();
  }

  if (!service) {
    fprintf(stderr, "The service is not defined!\n");
    usage();
  }

  if (!user) {
    fprintf(stderr, "The user is not defined!\n");
    usage();
  }

  if (!pwd) {
    fprintf(stderr, "The pwd is not defined!\n");
    usage();
  }

  if (ifsc_create(&handle, host, service)) {
    fprintf(stderr, "Error at creation %s\n", ifsc_get_last_errmsg(handle));
    exit(1);
  }
 
  if (trace)
    ifsc_set_trace(handle, IFS_SWITCH_ON);

  if (ifsc_connect(handle, user, pwd, &tradeid, &pid, &mmts_type)) {
    fprintf(stderr, "Error at connection %s\n", ifsc_get_last_errmsg(handle));
    exit(1);
  }

  while (1) {
    if (!marketbyprx)
      s = ifsc_get_next_orderbook(handle, &buff, &len, secboardid);
    else
      s = ifsc_get_next_marketbyprx(handle, &buff, &len, secboardid);

    if (s >= IFS_OK) {
      if (trace) fprintf (stderr, " orderbook queried for %s\n", secboardid);
      if (!s) fprintf(stdout, "%s\n", print_message(ptr_buf, buff, len)); 
      fflush(stdout);
    }  
    else
    {
      fprintf (stderr, " orderbook querying failed for %s\n", secboardid);
      fprintf (stderr, ifsc_get_last_errmsg(handle));
      return ifsc_disconnect(handle);
    }

    if (!cont)
      return ifsc_disconnect(handle);
      
    SLEEP(1);  
  }
    
  return ifsc_disconnect(handle);
}
