#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#ifndef WIN32
#  include <unistd.h>
#else 
#  include <wtypes.h>
#  include <winbase.h>
#  include <io.h>
#endif
#include "ifsapi.h"
#include "ifsdefs.h"
#include "ifsutil.h"



char *print(char *buf, char *msg, int len)
{
    int i;
    for (i = 0; i < len; i++){
    
        if (msg[i])
	    buf[i] = msg[i];
	else
	    buf[i] = '|';
    }
	
    buf[i]='\0';
    return buf;
}



void usage(void) 
{
  fprintf(stderr, "\r\nUsage: put_order \n");
  fprintf(stderr, "  <-1|-2>    using fields of MMTS1 or MMTS2 system\n");
  fprintf(stderr, "  -ffile   	output file for the order\n");
  fprintf(stderr, "  -t         trace\n");
  fprintf(stderr, "  -hhost     host name of server\n");
  fprintf(stderr, "  -sservice  service name of server\n");
  fprintf(stderr, "  -uuser     user name\n");
  fprintf(stderr, "  -ppwd      password\n");
  fprintf(stderr, "  -atrdaccid trade account id\n");  
  fprintf(stderr, "  -d<B|S>	buy or sell\n");  
  fprintf(stderr, "  -ysecid	security id\n");  
  fprintf(stderr, "  -vprice	price\n");
  fprintf(stderr, "  -qquantity	quantity\n");  
  fprintf(stderr, "  -h         prints this information\n");
  exit(1);
}
			  


int main (int argc, char *argv[])
{
  int i;
  int tradeid;
  int pid;
  int mmts_type;
  char * host = NULL;
  char * service = NULL;
  char * user = NULL;
  char * pwd = NULL;
  char * order_inp = NULL;
  char entrytype;
  FILE *f;
  IFSC_HDL handle;
  int mmts_mode = 0;
  int trace;
  
  char *trdAccId = NULL;
  char buySell = 0;
  char *boardId = NULL;
  char *secId = NULL;
  int price = 0;
  int quantity = 0;  
  
  int orderId = 0;
  int len = 0;
  char buf[1024];
  char *recBuff;
  char *lastValid;
  fixReal real;
  int seqNo = 0;
  int length = 0;    
  
  if (argc == 1) usage();
  
  for (i = 1; i < argc; i++){
    if (argv[i][0] == '-'){
      switch(argv[i][1]){
        case 'e':
          entrytype = *(argv[i] + 2); 
          break;
        case 't': 
          trace = 1; 
          break;
        case 'h':
          host = argv[i]+2;
          break;
        case 's':
          service = argv[i]+2;
          break;
        case 'u':
          user = argv[i]+2;
          break;
        case 'p':
          pwd = argv[i]+2;
          break;
        case 'f':
          order_inp = argv[i]+2;
          break;
        case '1':
          mmts_mode = 1;
          break;
        case '2':
          mmts_mode = 2;
          break;

	case 'a':	// TrdAccId
	    trdAccId = argv[i] + 2;	
	    break;
	case 'd':    	// BuySell
	    buySell = *(argv[i] + 2);
	    break;
	case 'b':	// BoardId
	    boardId = argv[i] + 2;
	    break;	    
	case 'y':	// SecId
	    secId = argv[i] + 2;
	    break;	    
	case 'v':	// Price
	    price = atoi(argv[i]+2);	
	    break;	    
	case 'q': 	// Quantity
	    quantity = atoi(argv[i]+2);		    
	    break;
	    
        case '?':
        case '-':
          usage();
          break;
      }
    } 
  }

  if (!mmts_mode) {
    fprintf(stderr, "The one of -1 (MMTS1) or -2 (MMTS2) should be set!");
    usage();
  }
  if (!host) {
    fprintf(stderr, "The host is not defined!\n");
    usage();
  }

  if (!service) {
    fprintf(stderr, "The service is not defined!\n");
    usage();
  }

  if (!user) {
    fprintf(stderr, "The user is not defined!\n");
    usage();
  }

  if (!pwd) {
    fprintf(stderr, "The pwd is not defined!\n");
    usage();
  }
  
  if (ifsc_create(&handle, host, service)) {
    fprintf(stderr, "Error at creation: %s\n", ifsc_get_last_errmsg(handle));
    exit(1);
  }

  if (trace)
    ifsc_set_trace(handle, IFS_SWITCH_ON);

  if (ifsc_connect(handle, user, pwd, &tradeid, &pid, &mmts_type)) {
    fprintf(stderr, "Error at connection: %s\n", ifsc_get_last_errmsg(handle));
    exit(1);
  }
  
  if (mmts_type != mmts_mode) {
    fprintf(stderr, "Error: mmts type of server connected to is differ from given in command line!");
    exit(1);
  }
  
  if (!trdAccId) {
    fprintf(stderr, "Error: trdAccId!\r\n");
    exit(1);
  }
  
  if (!buySell) {
    fprintf(stderr, "Error: buySell!\r\n");
    exit(1);  
  }
  
  if (!boardId) {
    fprintf(stderr, "Error: boardId!\r\n");
    exit(1);  
  }
  
  if (!secId) {
    fprintf(stderr, "Error: secId!\r\n");
    exit(1);  
  }
  
  if (!price) {
    fprintf(stderr, "Error: price!\r\n");
    exit(1);  
  }
  
  if (!quantity) {
    fprintf(stderr, "Error: quantity!\r\n");
    exit(1);  
  }
  
  if (mmts_mode == MMTS_TYPE_1) {    
  
      len = ifs_set_ids(buf, trdAccId);
      len += ifs_set_char(buf + len, buySell);
      len += ifs_set_string(buf + len, IFS_ORDERMETHOD_LEN, "01000000000000000100000010000000"); //32
      len += ifs_set_string(buf + len, IFS_BOARDID_LEN, boardId);
      len += ifs_set_ids(buf + len, secId);
      len += ifs_set_int(buf + len, price);
      len += ifs_set_int(buf + len, 0); 				// Yield OPTIONAL
      len += ifs_set_int(buf + len, quantity);
      len += ifs_set_int(buf + len, 0); 				// Hidden OPTIONAL
      len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, ""); 		// OPTIONAL 40
      len += ifs_set_int(buf + len, 20070209);				// OPTIONAL
      len += ifs_set_int(buf + len, 150000);					// OPTIONAL
      len += ifs_set_int(buf + len, 0);					// OPTIONAL
      len += ifs_set_int(buf + len, 0);					// OPTIONAL
      len += ifs_set_int(buf + len, 0);					// OPTIONAL
      len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, ""); 		// OPTIONAL 40
      
  } else {
  
      len = ifs_set_ids(buf, trdAccId);                                 // trdaccid
      len += ifs_set_enum(buf + len, buySell == 'B' ? BuySellVerb_Buy : BuySellVerb_Sell);          //buysell
      len += ifs_set_enum(buf + len, OrderType_Limit);			//OrderType  /*OrderType_Market*/
      len += ifs_set_enum(buf + len, Duration_Session);          	//Duration
      len += ifs_set_enum(buf + len, MMTS_True);          		//PurgeOnLogogg
      len += ifs_set_enum(buf + len, MMTS_True);          		//AllowSoftQtyLimit
      len += ifs_set_enum(buf + len, MMTS_True);          		//AllowSoftPriceLimit
      len += ifs_set_enum(buf + len, 0);		        	//PositionType
      len += ifs_set_enum(buf + len, MMTS_False);          		//IsPrivate
      len += ifs_set_string(buf + len, IFS_BOARDID_LEN, boardId);	// boardid
      len += ifs_set_string(buf + len, IFS_SEC_CODE_LEN, secId);      	// SecId
      real.value = price;
      real.decimals = 2;
      len += ifs_set_fix_real(buf + len, real);                		// price - OPTIONAL      
      real.value = NOT_DEFINED;
      real.decimals = NOT_DEFINED; 
      len += ifs_set_fix_real(buf + len, real);                  	// yield - OPTIONAL    
      len += ifs_set_int(buf + len, quantity);                 		// qty
      len += ifs_set_int(buf + len, NOT_DEFINED);			// visibleqty - OPTIONAL
      len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, "");	 	// brokerref
      len += ifs_set_int(buf + len, 0);                       		// exptime.date - OPTIONAL
      len += ifs_set_int(buf + len, 0);                   		// exptime.time - OPTIONAL      
      real.value = 0;
      real.decimals = 0; 
      len += ifs_set_fix_real(buf + len, real);				// triggerprice    
      len += ifs_set_string(buf + len, IFS_NAME_LEN, "");          	// traderef
      len += ifs_set_int(buf + len, 0);					// minfillqty
      len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, "");     	// internal ref 
  }

  if (!order_inp) {
    f = stdout;
    fprintf(stderr, "No output file was given, using stdout...\r\n");
  } else {
    f = fopen(order_inp, "wb");      
  }
  
  if ((ifsc_orderentry(handle, IFS_ACTION_ORDER_ADD, buf, len, &orderId)) == 0) {
  
    if (trace) fprintf (stderr, " order_add (orderid %d) sent\n", orderId);    
    
  } else {
  
    fprintf (stderr, " error at order_add - %s\n", ifsc_get_last_errmsg(handle));
  } 

  fwrite(print(buf, buf, len), len, sizeof(char), f);
  fclose(f);
  
  len = 0;

#ifndef WIN32
  sleep(2);
#else
  Sleep(2000);
#endif
  
  lastValid = (char*)malloc(1024*sizeof(char));
  
  while(ifsc_get_next_record(handle, &recBuff, &len, IFS_T_ORDERENTRY, &seqNo) == 0) {

	print(lastValid, recBuff, len);
  }
  
  printf("LAST OrderEntry: %X\r\n[%s]\r\n", lastValid, lastValid);
  free(lastValid);
	
  return ifsc_disconnect(handle);
}

