/****************************************
Ez az llomny kizrolag az Effice Kft. (2083 Solymr Toldi u. 21.) tulajdona,
kizrolag csak az Effice Kft. engedlyvel, az engedlyben meghatrozott clra
s mdon hasznlhat fel. A program ingyenesen trtnt tadsra, mkdsrt 
az Effice Kft. semmilyen felelssget nem vllal.

Interface client for socket based communication.
Example program for sending an mmorder.
****************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#ifndef WIN32
#  include <unistd.h>
#else 
#  include <wtypes.h>
#  include <winbase.h>
#  include <io.h>
#endif


#include "ifsapi.h"
#include "ifsdefs.h"
#include "ifsutil.h"

#define ORDER_BUF_SIZE 1024     /*+ Maximal size of the order buffer. +*/
#define SEP "|"                 /*+ Field separator character. +*/

static char Id[] = "send_mmorder ";

	/*+ trace flag +*/	
static int trace;

        /*+ System defined variable. +*/
extern int errno;
        /* Prototypes */
int main (int argc, char *argv[]);
static void order_mmts1(IFSC_HDL handle, char *line, int line_num);
static void order_mmts2(IFSC_HDL handle, char *line, int line_num);
static char * print_message(char * buf, char * msg, int len);
void usage( void);

/*+++++++++++++++++++++++++++++++++++++++++
Print a message.

static char * print_message Points to the output string.

char * buf Pointer to the print buffer.

char * msg Pointer to message.

int len Length of the input.
+++++++++++++++++++++++++++++++++++++++++*/
static char * print_message(char * buf, char * msg, int len)
{
  int i;
  for (i = 0; i < len; i++){
    if (msg[i]) 
      buf[i] = msg[i];
    else
      buf[i] = '|'; 
  }
  buf[i]='\0';
  return buf;
}
/*+++++++++++++++++++++++++++++++++++++++++
Prints usage information.
+++++++++++++++++++++++++++++++++++++++++*/
void usage( void){
  fprintf(stderr, "Usage <-1|-2> [-forders] [-t] [-hhost] [-sservice] [-uuser] [-ppwd] [-h]\n");
  fprintf(stderr, "  <-1|-2>    using fields of MMTS1 or MMTS2 system\n");
  fprintf(stderr, "  -forders   name of the file of input orders, if not given program reads from the stdin\n");
  fprintf(stderr, "  -t 	     trace\n");
  fprintf(stderr, "  -hhost     host name of server\n");
  fprintf(stderr, "  -sservice  service name of server\n");
  fprintf(stderr, "  -uuser     user name\n");
  fprintf(stderr, "  -pwd       password\n");
  fprintf(stderr, "  -h         prints this information\n");
  fprintf(stderr, "If no host, service, user, pwd defined then \n");
  fprintf(stderr, "  value of IFSHOST, IFSSERVICE, IFSHOST, IFSPWD environment used\n");
  exit(1);    
}

#define FN_MMORDER_MMTS1        19      /*+ Number of fields. +*/

#define FN_MMORDER_MMTS2        29      /*+ Number of fields. +*/

/*+++++++++++++++++++++++++++++++++++++++++
The parse_line function looks for the identifier in the options structure and writes the value into its variable.
Using MMTS1 fields.

IFSC_HDL Connection handle.

char *line Single line of the config file. The line should look like: identifier = value.

int line_num Line number of the input file.

This function takes a line and splits into fields of orders, then send that into pgw.
+++++++++++++++++++++++++++++++++++++++++*/
static void order_mmts1(IFSC_HDL handle, char *line, int line_num)
{
  char * cp;
  int i, len, s, orderid;
  char ptr_buf[0x8000];
  char buf[1024];
  int fieldnum;
  
  fieldnum = FN_MMORDER_MMTS1;

  // count number of fields
  cp = strchr(line, *SEP);
  for (i=1; i<fieldnum && cp && *cp+1; i++, cp++)
    cp = strchr(cp, *SEP);

  if (i<fieldnum){
    fprintf (stderr, "Missing field from line %d\n", line_num);
    return;
  }

  len = ifs_set_string(buf, IFS_BOARDID_LEN, strtok(line, SEP));            // board
  len += ifs_set_ids(buf + len, strtok(NULL, SEP));                         // sec
  len += ifs_set_char(buf + len, *strtok(NULL, SEP));                       // buysell
  len += ifs_set_char(buf + len, *strtok(NULL, SEP));                       // replace
  len += ifs_set_ids(buf + len, strtok(NULL, SEP));                         // buy trdacc
  len += ifs_set_string(buf + len, IFS_ORDERMETHOD_LEN, strtok(NULL, SEP)); // buy om
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));                   // buy price
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));                   // buy yield
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));                   // buy quentity
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));                   // buy hidden
  len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));   // buy brokerref
  len += ifs_set_ids(buf + len, strtok(NULL, SEP));                         // sell trdacc
  len += ifs_set_string(buf + len, IFS_ORDERMETHOD_LEN, strtok(NULL, SEP)); // sell om
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));                   // sell price
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));                   // sell yield
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));                   // sell quentity
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));                   // sell hidden
  len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));   // sell brokerref
  len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));   // internal ref

  if (trace) {
    print_message(ptr_buf, buf, len);
    fprintf(stderr, ">%s<\n",ptr_buf);
  }  

  if ((s = ifsc_mmorder(handle, buf, len, &orderid)) == 0) {
    if (trace) fprintf (stderr, " mmorder (orderid %d) sent\n", orderid);
  } 
  else
    fprintf (stderr, " error at mmorder - %s\n", ifsc_get_last_errmsg(handle));

  fprintf(stdout, "%d\n", orderid);  
}

/*+++++++++++++++++++++++++++++++++++++++++
The parse_line function looks for the identifier in the options structure and writes the value into its variable.
Using MMTS2 fields.

IFSC_HDL Connection handle.

char *line Single line of the config file. The line should look like: identifier = value.

int line_num Line number of the input file.

This function takes a line and splits into fields of orders, then send that into pgw.
+++++++++++++++++++++++++++++++++++++++++*/
static void order_mmts2(IFSC_HDL handle, char *line, int line_num)
{
  char * cp;
  int i, len, s, orderid;
  char ptr_buf[0x8000];
  char buf[1024];
  int fieldnum;
  fixReal real;
  
  fieldnum = FN_MMORDER_MMTS2;
  
  // count number of fields

  cp = strchr(line, *SEP);
  for (i=1; i<fieldnum && cp && *cp+1; i++, cp++)
    cp = strchr(cp, *SEP);

  if (i<fieldnum){
    fprintf (stderr, "Missing field from line %d\n", line_num);
    return;
  }
	  
      len = ifs_set_string(buf, IFS_BOARDID_LEN, strtok(line, SEP));			  // Boardid
      len += ifs_set_string(buf + len, IFS_SEC_CODE_LEN, strtok(NULL, SEP));      // SecId
	  len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));					  //OrderType
	  len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    //Duration
	  len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    //PurgeOnLogoff
	  len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    //AllowSoftQtyLimit
      len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    //AllowSoftPriceLimit
	  len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    //IsPrivate

	  /*
	  if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_int(buf + len, atoi(cp));                                  // Exptime.date
      else
      len += ifs_set_int(buf + len, NOT_DEFINED);
	  */
	   
	  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));                     // Exptime.date                                 
      len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));                     // Exptime.time

	  len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    //Buysell
	  len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    //Replace
	  len += ifs_set_ids(buf + len ,strtok(NULL, SEP));							  // Buytrdaccid

      real.value = atof(cp = strtok(NULL, SEP));
      real.decimals = ifs_is_empty_str(cp, IFS_MSG_LEN) ? NOT_DEFINED : 0;
      len += ifs_set_fix_real(buf + len, real);                                   // Buy price
      real.value = atof(cp = strtok(NULL, SEP));
      real.decimals = ifs_is_empty_str(cp, IFS_MSG_LEN) ? NOT_DEFINED : 0;
      len += ifs_set_fix_real(buf + len, real);                                   // Buy yield
      len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));                     // Buy qty
      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_int(buf + len, atoi(cp));                                  // Buy visibleqty
      else
      len += ifs_set_int(buf + len, NOT_DEFINED);
      len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));     // Buy brokerref
	  len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));					  //Buy PositionType
      len += ifs_set_string(buf + len, IFS_NAME_LEN, strtok(NULL, SEP));          // Buy traderef
	  len += ifs_set_ids(buf + len, strtok(NULL, SEP));							  // Selltrdaccid
      real.value = atof(cp = strtok(NULL, SEP));
      real.decimals = ifs_is_empty_str(cp, IFS_MSG_LEN) ? NOT_DEFINED : 0;
      len += ifs_set_fix_real(buf + len, real);                                   // Sell price

      real.value = atof(cp = strtok(NULL, SEP));
      real.decimals = ifs_is_empty_str(cp, IFS_MSG_LEN) ? NOT_DEFINED : 0;
      len += ifs_set_fix_real(buf + len, real);                                   // Sell yield

      len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));                     // Sell qty
      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_int(buf + len, atoi(cp));                                  // Sell visibleqty
      else
      len += ifs_set_int(buf + len, NOT_DEFINED);
      len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));     // Sell brokerref
	  len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));					  //Sell PositionType
      len += ifs_set_string(buf + len, IFS_NAME_LEN, strtok(NULL, SEP));          // Sell traderef
      len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));     // Internal ref

  if (trace) {
    print_message(ptr_buf, buf, len);
    fprintf(stderr, ">%s<\n",ptr_buf);
  }  

  if ((s = ifsc_mmorder(handle, buf, len, &orderid)) == 0) {
    if (trace) fprintf (stderr, " mmorder (orderid %d) sent\n", orderid);
  } 
  else
    fprintf (stderr, " error at mmorder - %s\n", ifsc_get_last_errmsg(handle));

  fprintf(stdout, "%d\n", orderid);  
}

/*+++++++++++++++++++++++++++++++++++++++++
This function reads a line from a file into a variable and returns the address of that variable.

static char * get_line A pointer to the line has been read from the config file.

FILE * file File pointer.

char * buffer Buffer area.

int length Length of the buffer.

Those line whose first character is # returned as zero length line. 
+++++++++++++++++++++++++++++++++++++++++*/
static char * get_line (FILE * file, char * buffer, int length)
{
  int i;
  static iseof = 0;

  if (iseof) return NULL;

  for (i = 0; i < length - 1; i++)
    {
      buffer[i] = getc (file);
      if (buffer[i] == 0xa || buffer[i] == EOF)
        break;
    }
  if (i == length - 1)
    fprintf (stderr, "%s -- line too long\n", buffer);

  if (buffer[i] == EOF)
    iseof = 1;
  buffer[i] = '\0';
  if (buffer[0] == '#')
    buffer[0] = '\0';
  return buffer; 
}

/*+++++++++++++++++++++++++++++++++++++++++
Socket based communication interface client. Clients can connect into this server to 
send mmorder request.

int main Tradition.

int argc Number of command line parameters.

char *argv[] Points to an array that contains the command line parameters.
+++++++++++++++++++++++++++++++++++++++++*/
int main (int argc, char *argv[])
{
  int i;
  int tradeid;
  int pid;
  int mmts_type;
  char * recvb;
  int orderid;
  char * host = NULL;
  char * service = NULL, * service1 = NULL, * service2 = NULL;
  char * user = NULL;
  char * pwd = NULL;
  char * order_inp = NULL;
  FILE *f;
  char buffer[ORDER_BUF_SIZE];
  IFSC_HDL handle;
  int mmts_mode = 0;

  host = getenv("IFSHOST");
  service1 = getenv("IFSSERVICE");
  service2 = getenv("IFS2SERVICE");
  user = getenv("IFSUSER");
  pwd = getenv("IFSPWD");

  if (argc == 1) usage();
  for (i = 1; i < argc; i++){
    if (argv[i][0] == '-'){
      switch(argv[i][1]){
        case 't': 
          trace = 1; 
          break;
        case 'h':
          host = argv[i]+2;
          break;
        case 's':
          service = argv[i]+2;
          break;
        case 'u':
          user = argv[i]+2;
          break;
        case 'p':
          pwd = argv[i]+2;
          break;
        case 'f':
          order_inp = argv[i]+2;
          break;
        case '1':
          mmts_mode = 1;
          break;
        case '2':
          mmts_mode = 2;
          break;
        case '?':
        case '-':
          usage();
          break;
      }
    } 
  }

  if (trace) fprintf (stderr, "%s Compiled: %s %s\n", Id, __DATE__, __TIME__);

  if (!mmts_mode) {
    fprintf(stderr, "The one of -1 (MMTS1) or -2 (MMTS2) should be set!");
    usage();
  }

  if (mmts_mode == 2 && service2)
    service = service2;
  else
    service = service1;

  if (!host) {
    fprintf(stderr, "The host is not defined!\n");
    usage();
  }

  if (!service) {
    fprintf(stderr, "The service is not defined!\n");
    usage();
  }

  if (!user) {
    fprintf(stderr, "The user is not defined!\n");
    usage();
  }

  if (!pwd) {
    fprintf(stderr, "The pwd is not defined!\n");
    usage();
  }
  
  if (ifsc_create(&handle, host, service)) {
    fprintf(stderr, "Error at creation: %s\n", ifsc_get_last_errmsg(handle));
    exit(1);
  }

  if (trace)
    ifsc_set_trace(handle, IFS_SWITCH_ON);
          
  if (ifsc_connect(handle, user, pwd, &tradeid, &pid, &mmts_type)) {
    fprintf(stderr, "Error at connection: %s\n", ifsc_get_last_errmsg(handle));
    exit(1);
  }
  
  if (mmts_type != mmts_mode) {
    fprintf(stderr, "Error: mmts type of server connected to is differ from given in command line!");
    exit(1);
  }

  if (order_inp)
    f = fopen (order_inp, "r");
  else
    f = stdin;  

  if (!f) {
    fprintf (stderr, strerror(errno));
    exit (-1);
  }

  for (i=1; get_line (f, buffer, ORDER_BUF_SIZE); i++) 
  {
    if (buffer[0] != '\0') 
      if (mmts_mode == 1)
        order_mmts1(handle, buffer, i);
      else
        order_mmts2(handle, buffer, i);
  }  

  fclose (f);

  return ifsc_disconnect(handle);
}
